@@ -265,7 +265,7 @@ class PlaceService(BaseRdbmsService):
         Find an administrative subdivision with an IP address
 
 
-        :param ip_address:  IPv4 address of the machine of a user.  It
+        :param ip_address: IPv4 address of the machine of a user.  It
             corresponds to a tuple consisting of four decimal numbers, each
             ranging from `0` to `255`.
 
@@ -301,6 +301,50 @@ class PlaceService(BaseRdbmsService):
         area = None if len(areas) == 0 else areas[0]
         return area
 
+
+    @staticmethod
+    def __find_area_with_location(
+            location,
+            area_level=None,
+            locale=None):
+        """
+        Find an administrative subdivision with a geographical location
+
+
+        :param location: An object `GeoPoint`.
+
+        :param area_level: The level of the administrative subdivision to
+            return a list of schools.  By default, it corresponds to a country
+            level.  The minimal area level is `AREA_LEVEL_DISTRICT`.
+
+        :param locale: An object `Locale` specifying the language to return
+            the name of the country.
+
+
+        :return: An object representing the information of the country found
+            with this IP address.
+        """
+        if area_level is None:
+            area_level = AREA_LEVEL_COUNTRY
+
+        # Search from the smallest possible administrative subdivision to
+        # improve a lot the performance (milliseconds v.s. a few seconds).
+        areas = AreaService().find_areas_with_location(
+            location,
+            lowest_area_level=min(AREA_LEVEL_DISTRICT, area_level),
+            highest_area_level=area_level,
+            locale=locale)
+
+        # Filter the country area.
+        areas = [
+            area
+            for area in areas
+            if area.area_level == area_level
+        ]
+
+        area = None if len(areas) == 0 else areas[0]
+        return area
+
     @staticmethod
     def __get_multiple_pixel_resolutions(bucket_name):
         """
@@ -1485,7 +1529,8 @@ class PlaceService(BaseRdbmsService):
                 row.get_object({
                     'account_id': cast.string_to_uuid,
                     'locale': cast.string_to_locale,
-                    'place_id': cast.string_to_uuid
+                    'place_id': cast.string_to_uuid,
+                    'property_name': AddressComponentName,
                 })
                 for row in cursor.fetch_all()
             ]
@@ -1738,9 +1783,9 @@ class PlaceService(BaseRdbmsService):
               one or more attributes of this place.
 
 
-:raise InvalidArgumentException: if no location has been found
-    arguments `area_id`, `location`, `bounding_box`, or
-    `client_ip_address`, has been passed to this function.
+        :raise InvalidArgumentException: if no location has been found
+            arguments `area_id`, `location`, `bounding_box`, or
+            `client_ip_address`, has been passed to this function.
         """
         # Validate arguments.
         if bounding_box and not isinstance(bounding_box, BoundingBox):
@@ -1759,21 +1804,39 @@ class PlaceService(BaseRdbmsService):
         if not isinstance(categories, (list, set, tuple)):
             categories = [categories]
 
+        # Determine which argument need to be used to filter places within a
+        # geographical area:
         #
-        if bounding_box:
-            geometry_wkt = self.__build_bounding_box_wkt(bounding_box)
-        elif not area_id and client_ip_address:
+        # 1. The identification of an administrative subdivision `area_id`
+        # 2. The location of a geographical area:
+        #    2.1. A circle defined with a center `location` and a `radius`
+        #    2.2. An administrative subdivision of level `area_level` that contains
+        #         the given `location`
+        # 3. An administrative subdivision of level `area_level` that contains
+        #    the location corresponding to the `client_ip_address`
+        if area_id:
+            location = None  # Clear this argument if defined to avoid conflict
+
+        elif location:
+            if area_level is not None:  # Prevalence over `radius`
+                area = self.__find_area_with_location(
+                    location,
+                    area_level=area_level,
+                    locale=locale)
+                if area is not None:
+                    area_id = area.area_id
+                    location = None  # Disable this search criteria
+
+        elif client_ip_address:
             area = self.__find_area_with_ip_address(
                 client_ip_address,
                 area_level=area_level,
                 locale=locale)
-
             if area is not None:
                 area_id = area.area_id
 
-#         location_provider_count = sum([provider is not None for provider in [ area_id, location, bounds, ip_address]])
-#         if keywords is None and location_provider_count != 1:
-#             raise self.InvalidArgumentException('One and only one geographical location provider MUST be provided')
+        if not area_id and not location:
+            raise ValueError('No geographical restriction is defined to filter places')
 
         # Find places that correspond to the specified criteria.
         with self.acquire_rdbms_connection(connection=connection) as connection:
@@ -1792,7 +1855,7 @@ class PlaceService(BaseRdbmsService):
                         AND (object_status = %(OBJECT_STATUS_ENABLED)s
                              OR (object_status = %(OBJECT_STATUS_PENDING)s AND account_id = %(account_id)s))
                         AND ((%(longitude)s IS NULL AND %(area_id)s IS NULL)
-                             OR (%(longitude)s IS NOT NULL AND ST_Distance_Sphere(
+                             OR (%(longitude)s IS NOT NULL AND ST_DistanceSphere(
                                      location,
                                      ST_SetSRID(ST_MakePoint(%(longitude)s, %(latitude)s), 4326)) <= %(radius)s)
                              OR (%(area_id)s IS NOT NULL AND ST_Contains(get_area_boundaries(%(area_id)s), location)))
@@ -1828,7 +1891,7 @@ class PlaceService(BaseRdbmsService):
                         AND (object_status = %(OBJECT_STATUS_ENABLED)s
                              OR (object_status = %(OBJECT_STATUS_PENDING)s AND account_id = %(account_id)s))
                         AND ((%(longitude)s IS NOT NULL 
-                              AND ST_Distance_Sphere(
+                              AND ST_DistanceSphere(
                                   location,
                                   ST_SetSRID(ST_MakePoint(%(longitude)s, %(latitude)s), 4326)) <= %(radius)s)
                              OR (%(area_id)s IS NOT NULL 
@@ -2213,7 +2276,7 @@ class PlaceService(BaseRdbmsService):
     #     """
     #     with self.acquire_rdbms_connection(auto_commit=True, connection=connection) as connection:
     #         cursor = connection.execute("""
-    #             SELECT ST_Distance_Sphere(
+    #             SELECT ST_DistanceSphere(
     #                      COALESCE(ST_3DClosestPoint(boundaries, ST_SetSRID(ST_MakePoint(%(longitude)s, %(latitude)s, %(altitude)s), 4326)),
     #                               location),
     #                      ST_SetSRID(ST_MakePoint(%(longitude)s, %(latitude)s, %(altitude)s), 4326)) AS distance
